function [data, model]=train(model, data)
%[data, model]=train(model, data)
% Simple linear classifier with Hebbian-style learning.
% Inputs:
% model     -- A hebbian learning object.
% data      -- A data object.
% Returns:
% model     -- The trained model.
% data      -- A new data structure containing the results.
% Usually works best with standardized data. Standardization is not
% performed here for computational reasons (we put it outside the CV loop).

% Isabelle Guyon -- isabelle@clopinet.com -- November 2010

if model.verbosity>0, fprintf('==> Training Hebbian classifier ... '); end
    
Posidx=find(data.Y>0);
Negidx=find(data.Y<0);

if pd_check(data)
    % Kernelized version
    model.W=zeros(1, length(data.Y));
    model.W(Posidx)=1/(length(Posidx)+eps);
    model.W(Negidx)=-1/(length(Negidx)+eps);
else
    n=size(data.X, 2);
    Mu1=zeros(1, n); Mu2=zeros(1, n);
    if ~isempty(Posidx)
        Mu1=mean(data.X(Posidx,:), 1);
    end
    if ~isempty(Negidx)
        Mu2=mean(data.X(Negidx,:), 1);
    end
    model.W=Mu1-Mu2; 
    B=(Mu1+Mu2)/2;
    model.b0=-model.W*B';
end

% Test the model
if model.test_on_training_data
    data=test(model, data);
end

if model.verbosity>0, fprintf('done\n'); end

