function d = test(a, d)
%d = test(a, d)
% Make a representation out of the closest cluster centers.
% Inputs:
% a -- A km model.
% d -- A data structure.
% Returns:
% d -- The same data structure, but X is replaced by the discriminant
% values.

% Isabelle Guyon -- September 2005 -- isabelle@clopinet.com
    
if isempty(a.mu)
    return;
end

[p, n]=size(d.X);
[pp, nn]=size(a.mu);
if n~=nn, error('Wrong dimensions'); end

if a.verbosity>0
    fprintf('==> Using km...\n');
end
if ~isempty(d.X)
    if a.verbosity>0
        fprintf('     Compute similarity measures (matrix %d x %d)\n', p, pp);
    end
    d.X = sv_dp('radial2',d.X,a.mu,0,a.gamma);
    if a.standardize
        d.X=(d.X-a.mean(ones(p,1),:))./a.sigma(ones(p,1),:);
    end
end

if a.verbosity>0
    fprintf('done\n');
end
return
