function [d, a]=train(a, d)
%[data, model]=train(model, data)
% Kmeans from a given algo or if not found random features.
% Inputs:
% a         -- A km learning object.
% d         -- A data structure.
% Returns:
% a         -- The trained model.
% d         -- A new data structure containing the preprocessed data.

% Isabelle Guyon -- isabelle@clopinet.com -- November 2010

if a.verbosity>0
    fprintf('==> Training km... ');
end

if exist('dumb_VQ', 'file')
    [a.mu, idx]=dumb_VQ(d.X, a.k);
elseif exist('uns_kmeans', 'file')
    [a.mu, idx]=uns_kmeans(d.X, a.k, a.itnum);
elseif isobject(kmeans)
    [dspider, aspider]=train(kmeans({['k=' num2str(a.k)]}), d); %use the spider version
    a.mu=aspider.mu;
    idx=dspider.X;
elseif exist('kmeans', 'file')
    [idx, a.mu]=kmeans(d.X, a.k);
else
    % Don't do anything, returns the original matrix
    a.mu=[];
end
   
if ~isempty(a.mu)
    d.X=a.mu(idx,:);
end

% Compute gamma
DP = sv_dp('euclid2',a.mu,a.mu);
a.gamma = 10/median(DP(:));

% Calculate standardization coefficients
if a.standardize
    a.mean = mean(X, 1);
    s = std(X,1,1); 
    s(s==0)=1;
    a.sigma=s;
end

if a.verbosity>0
    fprintf('\n... done: km trained.\n');
end