%==========================================================================
% LINEAR_TEMPLATE Linear classifier template object             
%==========================================================================
% a=linear_template(hyper) 
%
% This is an object similar to a Spider object
% http://www.kyb.mpg.de/bs/people/spider/
%
% Model: f(X) = X*W' + b0
%    W                   -- the weights
%    b0                  -- the bias
%
% Must also supports the kernelized version:
% Model: f(X) = K*W' + b0
% if the data supplied for training is a kernel matrix
% K=X*X' (semi-definite positive). The data supplied for
% testing must then be a matrix Xte*X'.
%
% You do not need to switch automatically to the kernelized version if the
% number of examples is smaller than the number of features, the learning
% curve program will do it.
%
% Hyperparameter:
%    verbosity              -- 0/1 flag
%    test_on_training_data  -- 0/1 flag (faster if 0)
%
% Methods:
%  train, test
%
% To create derived linear classifiers you should re-write the method
% "train" but you shoul not need to change "test".
% 
% Example:
% my_training_data=data_struct(X, Y);
% my_model=linear_template;
% % OR
% my_model=linear_template('verbosity=1');
% % OR
% my_model=linear_template({'verbosity=1', 'test_on_training_data=1'});
% [training_resu, trained_model]=train(my_model, my_training_data);
% my_test_data=data_struct(Xte, Yte);
% test_resu=test(my_model, my_test_data);
% auc(test_resu)
% roc(test_resu)

%Isabelle Guyon -- isabelle@clopinet.com -- November 2010

classdef linear_template
	properties (SetAccess = public)
        b0=0;
        W=[];
        verbosity=0;
        test_on_training_data=0;
    end
    methods
        %%%%%%%%%%%%%%%%%%%
        %%% CONSTRUCTOR %%%
        %%%%%%%%%%%%%%%%%%%
        function a = linear_template(hyper) 
            % Evaluate hyper-parameters entered with the syntax of the
            % Spider http://www.kyb.mpg.de/bs/people/spider/
            eval_hyper;
        end  
        
        function a = one_class_classifier(a, d)
            %a = one_class_classifier(a, d)
            % IMPORTANT: every linear classifier must also return a (good) 
            % classifier for a single training example or all training
            % examples from the same class. You can overload this method:
            a.W=[];
            lbl=unique(d.Y);
            if length(lbl)==1
                a.W=mean(d.X,1)*lbl;
            end
        end
        
    end %methods
end %classdef
  

 

 
 





