function [data, model]=train(model, data)
%[data, model]=train(model, data)
% Template linear classifier training method.
% Inputs:
% model     -- A linear classifier learning object.
% data      -- A structure created by data_struct with members data.X and data.Y.
%
% Returns:
% model     -- The trained model.
% data      -- A new data structure containing the results.

% Isabelle Guyon -- isabelle@clopinet.com -- November 2010

if model.verbosity>0, fprintf('==> Training %s... ', class(model)); end

% IMPORTANT: the method must also return a (good) classifier for a single
% training example or all examples from the same class. For example:
model=one_class_classifier(model, data);
if ~isempty(model.W), return; end

if pd_check(data)
% If the data matrix is a kernel matrix, 
% the follow the model f(X) = data.X*W' + b0, where 
% data.X is in fact a kernel matrix of the type X*X'.
%   [model.W, model.b0]=some_first_function(data.X, data.Y);
% Expect that the test data will be a matrix Xte*X'.
else
% Compute the weight and bias of the model f(X) = X*W' + b0
%   [model.W, model.b0]=some_other_function(data.X, data.Y);
end

% Test the model
if model.test_on_training_data
    data=test(model, data);
end

if model.verbosity>0, fprintf('done\n'); end


