%=============================================================================
% LSSVM Linear Least Square Support Vector Classifier object             
%=============================================================================  
% a=lssvm(hyper) 
%
% Generates a lssvm classication object.
%
% Model: f(X) = X*W' + b0
%    W                   -- the weights
%    b0                  -- the bias
%
% Hyperparameter:
%    verbosity              -- 0/1 flag
%    test_on_training_data  -- 0/1 flag (faster if 0)
%
% Methods:
%  train, test

% ========================================================
% Adapted from Gavin Cawley:
% Uses the virtual leave-one-out method for the LS-SVM.
% At the moment it needs the fminunc 
% routine from the MATLAB optimisation toolbox.
% ========================================================

classdef lssvm < linear_template
    
    properties (SetAccess = public)
        coef0=    0;
        degree=   1;
        gamma=    0;
        shrinkage=[];
        balance=  1;
        optimizer='fminunc'; 
        alpha=[];
        Xsv=[];
        ber_loo=[];
        kernel='linear'; % We authorize only linear kernel
    end
    
     % This is a derived class
     methods
        %%%%%%%%%%%%%%%%%%%
        %%% CONSTRUCTOR %%%
        %%%%%%%%%%%%%%%%%%%
        function a = lssvm(hyper) 
            if nargin<1, hyper=[]; end
            a=a@linear_template(hyper);
        end         
    end
end
  

















