function [data, model] =  train(model, data)
%[data, model] =  train(model, data)
% Train a least square support vector classifier.
% kernels supported in general (here only linear kernel supported):
%    linear                             k(x,y)=x.y
%    poly                poly degree d, k(x,y)=(x.y+1)^d
%    rbf                 sigma,         k(x,y)=exp(-|x-y|^2/(2*sigma^2))
%    poly_rbf            const, degree d, gamma
%                        k(x,y)=(x.y+const)^d exp(-gamma|x-y|^2)*sigma^2))
% Note: poly_rbf includes all the other kernels for special values
%       of the kernel parameters.
% Inputs:
% model -- A lssvm classifier object.
% data -- A training data object.
% Returns:
% data -- The same data structure, but X is replaced by the class label
% predictions on training data.
% model -- The trained lssvm classifier.

% Isabelle Guyon -- September 2006-November 2010 -- isabelle@clopinet.com

if model.verbosity>0
    disp(['training ' class(model) '... '])
end

model=one_class_classifier(model, data);
if ~isempty(model.W), return; end

opt=model.optimizer;
[p, n]=size(data.X);

switch opt
    case 'fminunc'
        % Note: model can be an object or a plain structure.
        [data, model]=lssvm_demo(model, data);
    otherwise
        disp 'Wrong option'
end

if pd_check(data)
    model.W=model.alpha';
else
    model.W=model.alpha'*model.Xsv;
end
model.Xsv=[]; % Save memory

if model.verbosity>0, fprintf('done\n'); end

 