function posdef = pd_check(A)
%chol = pd_check(A)
% Check whether a matrix is positive semi-definite

% Because it breaks down for large matrices, we check for 10 random
% 100*100 submatrices.
% We have a pretty lenient PD check because we do not want to miss any
% kernel matrices.

debug=0;
[n, n1] = size(A); 
posdef = 0; 

if n~=n1, 
    if debug, fprintf('pd_check: Matrix not square!\n'); end
    return; 
end
if any(A-A'), 
    if debug, fprintf('pd_check: Matrix not symmetric!\n'); end
    return; 
end

if n>100
    for k=1:10
        rp=randperm(n);
        RP(k,:)=rp(1:100);
    end
else
    RP(1,:)=1:n;
end

tol = -1e-15;

for k=1:size(RP, 1)
    idx=RP(k,:);
    a=A(idx,idx);
    D=diag(a);
    M=max(D);
    if M<=0, return; end
    f=min(D)/M;
    if f < tol, return; end
end

posdef = 1;  