function [Xtrain_norm, Xtest_norm] = standardize_data(Xtrain, Xtest, add_fudge)
%[Xtrain_norm, Xtest_norm] = standardize_data(Xtrain, Xtest, add_fudge)
% Normalize the training and test data
% by shifting the mean of Xtrain to zero and rescaling the stdev to one.
% If Xtest is specified, the same shift and scaling are applied to Xtest.
% Note: sparse matrices are converted to full matrices.

% Isabelle Guyon, last change September 2003, isabelle@clopinet.com
% Changed std to divide std + fudge factor.

if nargin<2, Xtest=[]; end
if nargin<3, add_fudge=1; end

Mean = mean(Xtrain);
Stdev = std(Xtrain);
Stdev(Stdev==0)=1;
if add_fudge
    fudge=median(Stdev);
else
    fudge=0;
end

p=size(Xtrain, 1);
Xtrain=full(Xtrain);
Xtrain_norm=zeros(size(Xtrain));
for i=1:p
   Xtrain_norm(i,:) = (Xtrain(i,:)-Mean)./(Stdev+fudge+eps);
end

if ~isempty(Xtest)
    Xtest=full(Xtest);
    Xtest_norm=zeros(size(Xtest));
	[p,n]=size(Xtest);
	for i=1:p
	   Xtest_norm(i,:) = (Xtest(i,:)-Mean)./(Stdev+fudge+eps);
	end
end

