% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-
%                               SAMPLE CODE FOR THE "
%                 "UNSUPERVISED AND TRANSFER LEARNING CHALLENGE"
% -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% DISCLAIMER: ALL INFORMATION, SOFTWARE, DOCUMENTATION, AND DATA ARE PROVIDED "AS-IS" 
% ISABELLE GUYON AND/OR OTHER CONTRIBUTORS DISCLAIM ANY EXPRESSED OR IMPLIED WARRANTIES, 
% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
% FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY OF NON-INFRIGEMENT OF ANY
% THIRD PARTY'S INTELLECTUAL PROPERTY RIGHTS. IN NO EVENT SHALL ISABELLE GUYON 
% AND/OR OTHER CONTRIBUTORS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES 
% OR ANY DAMAGES WHATSOEVER ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
% OF SOFTWARE, DOCUMENTS, MATERIALS, PUBLICATIONS, OR INFORMATION MADE AVAILABLE FOR 
% THE CHALLENGE. 

% Author: Isabelle Guyon -- December 2010 -- ul@clopinet.com

close all;
fclose all;
clear all;
%clear classes;
%clc

%% -o-|-o-|-o-|-o-|-o-|-o-|-o- BEGIN USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

% 1) Directory tree (you do not need to change that)
% -----------------
%       The present set up supposes the main.m file is in a 
% directory <dir.root> from which you are running at the Matlab prompt:
%
% > main;

dir.root     = '.';  % Change that to the directory of your project

% Get the absolute path
this_dir=pwd; cd(dir.root); dir.root=pwd; cd(this_dir); 

% You must download the data and put it in the Data directory!
% We assume that the .mat files are in subdirectories names with the
% dataset name in capital letters. e.g. ule_devel.mat is in ULE/
% On some computers, when using Windows, replace '/' by '\'.
dir.data    = [dir.root '/Data'];      % Directory of the data files
dir.matlab  = [dir.root '/Matlab'];    % Directory of the sample functions and machine learning (ML) objects
dir.clop    = [dir.root '/CLOP'];      % Directory whre to put CLOP ML objects (optional) see http://clopinet.com/CLOP
dir.prepro  = [dir.root '/Prepro'];    % Directory of the resulting preprocessed data
dir.zip     = [dir.root '/Zipped'];    % ** Zipped files ready to go!! ** (zips of prepro)
dir.resu    = [dir.root '/Results'];   % Directory of the results (only for toy data)

%% 2) Identify your experiment
% ----------------------------
% The nickname you used on the challenge website
% (it does not matter if you choose something else here)
my_workbench_id = 'myname';  

% The date (if you want a more precise date and time, change that)
the_date        = datestr(now, 'yyyy-mm-dd');  

%% 2) Setup of your experiment
% ----------------------------
% The list of datasets to work on:
dataset     = {'ule', 'avicenna', 'harry', 'rita', 'sylvester', 'terry'}; 
       
% The name of the machine learning algorithms used for unsupervised or
% transfer learning. In this example, 'raw' is the raw data, 
% 'norma' is the raw data after normalization, 'km' is kmeans clustering, 
% and 'sp'is some polynomial feature expansion creating many features 
% and causing the sample code 'kernelize' the data.
% Define your own model!
prepro_algo     = {'km', 'raw', 'norma', 'sp'};

% The list of data subsets to work on (during development, omitting the 
% final evaluation set, saves time and creates smaller submissions).
set_name={'devel', 'valid', 'final'};

% Choose whether to zip only the validation data if the files are big
zip_final_only_if_small=1;

%% -o-|-o-|-o-|-o-|-o-|-o-|-o- END USER-PREFERENCES -o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-

%% Initializations           
warning off; addpath(dir.root); addpath(dir.matlab); addpath([dir.matlab '/Functions']); addpath(dir.clop); warning on
if ~exist('whoisclop', 'file') && exist('use_spider_clop.m', 'file'), use_spider_clop(dir.clop); end
if ~exist(dir.data, 'dir') == 7, fprintf('[No data directory!]\n');  return ; end
makedir(dir.resu);
makedir(dir.prepro);
makedir(dir.zip);
debug=0;
warning off; 
% Learning curve setting used for evaluation:
alg=hebbian;     % linear model we use for the evaluation on the challenge platform
                 % (we also provide lssvm with the sample code)
min_repeat=10;   % min. num. repeats for each point on learning curve
max_repeat=500;  % max. num. repeats for each point on learning curve
ebar=0.01;       % max. desired error bar
max_point_num=7; % maximum number of points on the learning curve

%% LOOP OVER DATASETS 
% ===================
warning on
for k=1:length(dataset)
    
    data_name   = dataset{k};
    the_data_dir=[dir.data sl upper(data_name) sl];
    
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n');
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(data_name));
    fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-|-o-\n\n'); 
    
    % Load the data
    [X0, Y, p]=create_data_structure(the_data_dir, data_name, set_name);
    if isempty(p), fprintf('==> %s: No data, skip this one!\n', upper(data_name)); continue; end
    
    %% LOOP OVER ALGORITHMS 
    % =====================
    for h=1:length(prepro_algo)
        palg=prepro_algo{h};
        fprintf('\n-o-|-o-|-o-|-o-|-o-|-o-      %s      -o-|-o-|-o-|-o-|-o-|-o-\n', upper(palg));
        prepro_name  = [my_workbench_id '_' data_name '_' palg '_' the_date];
        
        if strcmp(palg, 'raw')
            X=X0;
        else
            tic
            if ~strcmp(palg, 'norma')
                % Preprocess the data using your model
                X=preprocess_data(palg, X0, Y.devel); 
            end
            % You may also want to standardize and/or normalize the data
            X=my_normalize(X);
            X=my_standardize(X);
            fprintf('==> Data preprocessed in %g seconds\n', toc);
        end      

        % If your number of features n exceeds the number of examples m, 
        % replace your data matrices X (m, n) by the smaller matrix X*X'
        % of dimension (m, m) -- we'll notice automatically:
        if size(X.valid, 2)>size(X.valid, 1)
            X=kernelize(X);
        end

        % Optionally, quantize data (precision is usually unnecessary and this makes the submissions a lot smaller)
        X=quantize_data(X);

        % Save the data (note: the name convention is flexible but files must
        % contain the dataset name and end by _valid.prepro or _final.prepro.
       	save_data(dir.prepro, prepro_name, X);

        % Create a zip file with the query in the Queries directory
       	make_zip(dir.prepro, dir.zip, prepro_name, zip_final_only_if_small);        

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%% The zip files must be submitted to the web site %%%%%%
        %%%%%%% IMPORTANT: do not submit preprocessed data for  %%%%%%
        %%%%%%% the development set (too big and not evaluated).%%%%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % For the toy example, you can run the scoring software yourself:
        sname=setdiff(set_name, 'devel');
        for i=1:length(sname)
            
            tic
            % Get the truth values (labels)
            Y.(sname{i})=load([the_data_dir data_name '_' sname{i} '.label']);
            if isempty(Y), continue; end % Abort if no truth values available
            
            full_resu_name=[prepro_name '_' class(alg) '_' sname{i}];
            resu_name=[data_name '_' palg '_' class(alg) '_' sname{i}];
            fprintf('==> Making %s learning curve\n', resu_name);

            % Make the learning curve
            [x, y, e] =make_learning_curve(alg, X.(sname{i}), Y.(sname{i}), min_repeat, max_repeat, ebar, max_point_num);

            % Compute the (normalized) area under the learning curve
            score=alc(x, y);
            fprintf('==> Normalized ALC for %s  = %5.4f\n', resu_name, score);
            fprintf('==> Learning curve completed in %g seconds\n', toc);
            
            % Plot the learning curve
            hh=plot_learning_curve(resu_name, score, x, y, e);

            % Save the results
            save([dir.resu sl full_resu_name '.txt'], 'x', 'y', 'e', '-ascii' );
            saveas(hh, [dir.resu sl full_resu_name '.png']);
        end
    end     % for loop over preprocessings
end         % for loop over datasets

%%%%%%%%%%%%%%%%%%%%% END MAIN %%%%%%%%%%%%%%%%%%%%%%%
